import datetime

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.html import mark_safe
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _


class Vocabulary(models.Model):
    name = models.CharField(_("Name"), max_length=255, unique=True)
    slug = models.SlugField(
        _("Slug"),
        unique=True,
        help_text=_("slug is human-readable, to make referencing easier"),
    )

    def __str__(self):
        return self.name

    class Meta:
        ordering = ["name"]
        verbose_name = _("Vocabulary")
        verbose_name_plural = _("Vocabularies")


class Term(models.Model):
    vocabulary = models.ForeignKey(Vocabulary, null=False, on_delete=models.CASCADE)
    vocabulary.verbose_name = _("Vocabulary")
    term = models.CharField(_("Term"), max_length=255)
    slug = models.CharField(_("Slug"), max_length=255, unique=True, blank=True)

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = self.vocabulary.slug + "__" + slugify(self.term)
        super().save(*args, **kwargs)

    def __str__(self):
        return self.term

    class Meta:
        verbose_name = _("Term")
        verbose_name_plural = _("Terms")
        order_with_respect_to = "vocabulary"


class Note(models.Model):
    note = models.TextField()
    user = models.ForeignKey(
        get_user_model(), on_delete=models.SET_NULL, null=True, editable=False
    )
    created = models.DateTimeField(auto_now=True)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey("content_type", "object_id")

    def __str__(self):
        ret = f"{self.note}<br/><small><i>{(self.created or datetime.datetime.now() ).date()}"
        if self.user:
            ret += f" - {self.user}"
        ret += "</i></small>"
        return mark_safe(ret)

    class Meta:
        ordering = ["-created"]



from bread import views as breadviews
from bread.utils.urls import autopath, default_model_paths, model_urlname, reverse_model
from bread.views import AddView, BrowseView, EditView
from django.views.generic import RedirectView

import basxconnect.core.views.tag_views
from basxconnect.core.views import settings_views
from basxconnect.core.views.person import (
    person_browse_views,
    person_details_views,
    person_modals_views,
    search_person_view,
)

from . import models
from .views.relationship_views import AddRelationshipView, EditRelationshipView
from .views.term import TermsBrowseView
from .views.vocabulary import VocabularyBrowseView
from .wizards.add_person import AddPersonWizard

urlpatterns = [
    autopath(
        RedirectView.as_view(
            url=reverse_model(models.Person, "addwizard", kwargs={"step": "Search"})
        ),
        model_urlname(models.Person, "add"),
    ),
    autopath(
        RedirectView.as_view(url=reverse_model(models.Person, "browse")),
        model_urlname(models.NaturalPerson, "browse"),
    ),
    autopath(
        RedirectView.as_view(url=reverse_model(models.Person, "browse")),
        model_urlname(models.LegalPerson, "browse"),
    ),
    autopath(
        RedirectView.as_view(url=reverse_model(models.Person, "browse")),
        model_urlname(models.PersonAssociation, "browse"),
    ),
    autopath(
        AddPersonWizard.as_view(url_name=model_urlname(models.Person, "addwizard")),
        model_urlname(models.Person, "addwizard"),
    ),
    *default_model_paths(
        models.Person,
        browseview=person_browse_views.PersonBrowseView,
        deleteview=breadviews.DeleteView._with(softdeletefield="deleted"),
    ),
    *default_model_paths(
        models.NaturalPerson,
        editview=person_details_views.NaturalPersonEditView,
        readview=person_details_views.NaturalPersonReadView,
        deleteview=breadviews.DeleteView._with(softdeletefield="deleted"),
        copyview=breadviews.generate_copyview(
            models.NaturalPerson,
            attrs={
                "personnumber": models.random_personid,
                "primary_postal_address": None,
                "primary_email_address": None,
            },
            labelfield="name",
            copy_related_fields=(
                "core_web_list",
                "core_email_list",
                "core_phone_list",
                "core_fax_list",
                "core_postal_list",
            ),
        ),
    ),
    *default_model_paths(
        models.LegalPerson,
        editview=person_details_views.LegalPersonEditView,
        readview=person_details_views.LegalPersonReadView,
        deleteview=breadviews.DeleteView._with(softdeletefield="deleted"),
        copyview=breadviews.generate_copyview(
            models.LegalPerson,
            attrs={
                "personnumber": None,
                "primary_postal_address": None,
                "primary_email_address": None,
            },
            labelfield="name",
            copy_related_fields=(
                "core_web_list",
                "core_email_list",
                "core_phone_list",
                "core_fax_list",
                "core_postal_list",
            ),
        ),
    ),
    *default_model_paths(
        models.PersonAssociation,
        editview=person_details_views.PersonAssociationEditView,
        readview=person_details_views.PersonAssociationReadView,
        deleteview=breadviews.DeleteView._with(softdeletefield="deleted"),
        copyview=breadviews.generate_copyview(
            models.PersonAssociation,
            attrs={
                "personnumber": None,
                "primary_postal_address": None,
                "primary_email_address": None,
            },
            labelfield="name",
            copy_related_fields=(
                "core_web_list",
                "core_email_list",
                "core_phone_list",
                "core_fax_list",
                "core_postal_list",
            ),
        ),
    ),
    *default_model_paths(
        models.Relationship,
        editview=EditRelationshipView,
        addview=AddRelationshipView,
    ),
    *default_model_paths(models.RelationshipType),
    *default_model_paths(
        models.Term,
        addview=AddView._with(fields=["term", "vocabulary"]),
        editview=EditView._with(fields=["term", "slug"]),
        browseview=TermsBrowseView._with(
            rowclickaction=BrowseView.gen_rowclickaction("edit", return_to_current=True)
        ),
    ),
    *default_model_paths(
        models.Vocabulary,
        browseview=VocabularyBrowseView,
    ),
    *default_model_paths(models.Postal),
    *default_model_paths(models.Phone),
    *default_model_paths(models.Web),
    autopath(settings_views.generalsettings),
    autopath(
        person_details_views.togglepersonstatus,
        model_urlname(models.Person, "togglestatus"),
    ),
    autopath(settings_views.personsettings),
    autopath(settings_views.relationshipssettings),
    autopath(search_person_view.searchperson),
    autopath(
        person_modals_views.NaturalPersonEditMailingsView.as_view(),
        urlname=model_urlname(models.NaturalPerson, "ajax_edit_mailings"),
    ),
    autopath(
        person_modals_views.LegalPersonEditMailingsView.as_view(),
        urlname=model_urlname(models.LegalPerson, "ajax_edit_mailings"),
    ),
    autopath(
        person_modals_views.NaturalPersonEditPersonalDataView.as_view(),
        urlname=model_urlname(models.NaturalPerson, "ajax_edit_personal_data"),
    ),
    autopath(
        person_modals_views.LegalPersonEditPersonalDataView.as_view(),
        urlname=model_urlname(models.LegalPerson, "ajax_edit_personal_data"),
    ),
    autopath(
        person_modals_views.PersonAssociationEditPersonalDataView.as_view(),
        urlname=model_urlname(models.PersonAssociation, "ajax_edit"),
    ),
    autopath(
        person_modals_views.EditPostalAddressView.as_view(),
        urlname=model_urlname(models.Postal, "ajax_edit"),
    ),
    autopath(
        person_modals_views.AddPostalAddressView.as_view(),
        urlname=model_urlname(models.Postal, "ajax_add"),
    ),
    autopath(
        EditView._with(fields=["remarks"]).as_view(model=models.Person),
        urlname=model_urlname(models.Person, "ajax_edit_remarks"),
    ),
    autopath(
        EditView._with(fields=["tags"]).as_view(model=models.Person),
        urlname=model_urlname(models.Person, "ajax_edit_tags"),
    ),
    *[
        autopath(
            EditView._with(model=m, fields=["personnumber", "type"]).as_view(),
            urlname=model_urlname(m, f"{m._meta.model_name}_ajax_edit_metadata"),
        )
        for m in [models.NaturalPerson, models.LegalPerson, models.PersonAssociation]
    ],
    autopath(
        person_details_views.confirm_delete_email,
        urlname=model_urlname(models.Email, "delete"),
    ),
    autopath(
        person_modals_views.AddEmailAddressView.as_view(),
        urlname=model_urlname(models.Email, "add"),
    ),
    autopath(
        person_modals_views.EditEmailAddressView.as_view(),
        urlname=model_urlname(models.Email, "edit"),
    ),
    autopath(
        basxconnect.core.views.tag_views.bulk_tag_operation_view,
        urlname=model_urlname(models.Person, "bulk-tag-operation"),
    ),
    autopath(
        basxconnect.core.views.tag_views.AddTagView.as_view(),
        urlname=model_urlname(models.Term, "ajax_add"),
    ),
]



# Generated by Django 3.2.4 on 2022-01-05 07:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mailer_integration", "0019_auto_20211210_1602"),
    ]

    operations = [
        migrations.AddField(
            model_name="subscription",
            name="deleted",
            field=models.BooleanField(
                blank=True, default=False, editable=False, verbose_name="Deleted"
            ),
        ),
    ]



import htmlgenerator as hg
from bread import layout
from django.utils.translation import gettext_lazy as _

R = layout.grid.Row
C = layout.grid.Col


def editperson_toolbar(request):
    deletebutton = layout.button.Button(
        _("Delete"),
        buttontype="ghost",
        icon="trash-can",
        notext=True,
        **layout.aslink_attributes(
            hg.F(lambda c: layout.objectaction(c["object"], "delete"))
        ),
    )
    restorebutton = layout.button.Button(
        _("Restore"),
        buttontype="ghost",
        icon="undo",
        notext=True,
        **layout.aslink_attributes(
            hg.F(
                lambda c: layout.objectaction(
                    c["object"], "delete", query={"restore": True}
                )
            )
        ),
    )
    copybutton = layout.button.Button(
        _("Copy"),
        buttontype="ghost",
        icon="copy",
        notext=True,
        **layout.aslink_attributes(
            hg.F(lambda c: layout.objectaction(c["object"], "copy"))
        ),
    )

    return hg.SPAN(
        hg.If(hg.C("object.deleted"), restorebutton, deletebutton),
        copybutton,
        layout.button.PrintPageButton(buttontype="ghost"),
        _class="no-print",
        style="margin-bottom: 1rem; margin-left: 1rem",
        width=3,
    )


def editperson_head(request):
    return hg.BaseElement(
        R(
            C(
                hg.H3(
                    hg.SPAN(
                        hg.C("object"),
                        style=hg.If(
                            hg.C("object.deleted"), "text-decoration: line-through"
                        ),
                    ),
                    editperson_toolbar(request),
                ),
                width=12,
            ),
            style="padding-top: 1rem",
        ),
    )



from celery_haystack.indexes import CelerySearchIndex
from haystack import indexes

from . import models


class PersonIndex(CelerySearchIndex, indexes.Indexable):
    text = indexes.CharField(document=True, model_attr="search_index_snippet")
    personnumber = indexes.CharField(model_attr="personnumber")
    name_auto = indexes.EdgeNgramField(model_attr="search_index_snippet")

    def get_model(self):
        return models.Person

    def index_queryset(self, using=None):
        return models.Person.objects.not_deleted()

    def read_queryset(self, using=None):
        return models.Person.objects.not_deleted()



import django.db.models
import htmlgenerator as hg
from bread import layout
from bread.layout import ObjectFieldValue
from bread.layout.components.datatable import DataTableColumn
from bread.layout.components.icon import Icon
from bread.layout.components.modal import modal_with_trigger
from bread.utils import Link, ModelHref, pretty_modelname, reverse_model
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from htmlgenerator import mark_safe

from basxconnect.core import models
from basxconnect.core.layouts.editperson.common.utils import tile_with_icon, tiling_col

R = layout.grid.Row
C = layout.grid.Col


def numbers(request):
    return tile_with_datatable(
        models.Phone,
        hg.F(lambda c: c["object"].core_phone_list.all()),
        ["type", "number"],
        request,
    )


def tile_with_datatable(model, queryset, columns, request):
    addmodal = layout.modal.Modal.with_ajax_content(
        _("Add"),
        ModelHref(
            model,
            "add",
            query=hg.F(lambda c: {"person": c["object"].pk, "asajax": True}),
        ),
        submitlabel=_("Save"),
    )

    def editmodal():
        return hg.F(
            lambda c: modal_with_trigger(
                layout.modal.Modal.with_ajax_content(
                    _("Edit"),
                    ModelHref.from_object(hg.C("row"), "edit", query={"asajax": True}),
                    submitlabel=_("Save"),
                ),
                layout.button.Button,
                notext=True,
                small=True,
                buttontype="ghost",
                icon="edit",
                label=_("Edit"),
                _class="bx--overflow-menu",
            )
        )

    return tiling_col(
        layout.datatable.DataTable.from_queryset(
            queryset,
            model=model,
            prevent_automatic_sortingnames=True,
            columns=columns,
            rowactions=[
                editmodal(),
                Link(
                    href=ModelHref(
                        model,
                        "delete",
                        kwargs={"pk": hg.C("row.pk")},
                        query={"next": request.get_full_path()},
                    ),
                    iconname="trash-can",
                    label=_("Delete"),
                ),
            ],
            primary_button=layout.button.Button(
                _("Add"), buttontype="primary", **addmodal.openerattributes
            ),
            style="border-top: none;",
        ),
        addmodal,
    )


def email(request):
    return tile_with_datatable(
        models.Email,
        hg.F(lambda c: c["object"].core_email_list.all()),
        [
            DataTableColumn(
                layout.ObjectFieldLabel("type", models.Email),
                hg.SPAN(
                    hg.C("row.type"),
                    hg.If(
                        hg.F(
                            lambda c: c["row"] == c["row"].person.primary_email_address
                        ),
                        hg.BaseElement(" (", _("primary"), ")"),
                        "",
                    ),
                ),
            ),
            "email",
        ],
        request,
    )


def urls(request):
    return tile_with_datatable(
        models.Web,
        hg.F(lambda c: c["object"].core_web_list.all()),
        ["type", "url"],
        request,
    )


def edit_heading(model: type):
    return _("Edit %s") % pretty_modelname(model)


def display_postal():
    postal = hg.C("i")
    modal = layout.modal.Modal.with_ajax_content(
        heading=edit_heading(models.Postal),
        url=ModelHref(
            models.Postal,
            "ajax_edit",
            kwargs={"pk": postal.pk},
            query={"asajax": True},
        ),
        id=hg.format("postal-modal-{}", hg.C("i").id),
        submitlabel=_("Save"),
    )
    is_inactive = hg.F(
        lambda c: c["i"].valid_until and c["i"].valid_until < timezone.now().date()
    )
    return R(
        C(
            hg.DIV(
                postal.type,
                hg.If(
                    hg.F(
                        lambda c: c["i"].person.primary_postal_address
                        and c["i"].person.primary_postal_address.pk == c["i"].pk
                    ),
                    hg.BaseElement(" (", _("primary"), ")"),
                ),
                style="font-weight: bold; margin-bottom: 1rem;",
            ),
            hg.DIV(
                ObjectFieldValue("address", object_contextname="i"),
                style="margin-bottom: 0.25rem;",
            ),
            hg.DIV(postal.postcode, " ", postal.city, style="margin-bottom: 0.25rem;"),
            hg.DIV(postal.get_country_display()),
            hg.If(
                postal.valid_from,
                hg.DIV(
                    hg.SPAN(_("Valid from: "), style="font-weight: bold;"),
                    ObjectFieldValue("valid_from", object_contextname="i"),
                    " ",
                    style="display: inline-block; margin-top: 1rem; margin-right: 1rem;",
                ),
                hg.BaseElement(),
            ),
            hg.If(
                postal.valid_until,
                hg.DIV(
                    hg.SPAN(_("Valid until: "), style="font-weight: bold;"),
                    ObjectFieldValue("valid_until", object_contextname="i"),
                    style="display: inline-block; margin-top: 1rem;",
                ),
                hg.BaseElement(),
            ),
            hg.DIV(
                edit_postal_button(modal),
            ),
        ),
        _class=hg.If(is_inactive, "inactive_postal", ""),
        style=hg.If(
            is_inactive,
            "display: none; margin-top: 1.5rem; color: #a8a8a8;",
            "margin-top: 1.5rem;",
        ),
    )


def edit_postal_button(modal):
    return hg.SPAN(
        layout.button.Button(
            "",
            buttontype="ghost",
            icon="edit",
            **modal.openerattributes,
        ),
        modal,
    )


def delete_postal_button(postal):
    return layout.button.Button(
        _("Delete"),
        buttontype="ghost",
        icon="trash-can",
        notext=True,
        hx_post=reverse_model(
            models.Postal,
            "ajax_delete",
            kwargs={
                "pk": postal.pk,
            },
            query={
                "asajax": True,
            },
        ),
    )


def postals():
    return tile_with_icon(
        Icon("map"),
        hg.H4(_("Address(es)")),
        R(
            C(
                hg.Iterator(
                    hg.F(
                        lambda c: getattr(c["object"], "core_postal_list")
                        .order_by(
                            django.db.models.F("valid_until").desc(nulls_first=True)
                        )
                        .all()
                        if hasattr(c["object"], "core_postal_list")
                        else []
                    ),
                    "i",
                    display_postal(),
                )
            )
        ),
        R(
            C(
                layout.button.Button(
                    _("Hide inactive postals"),
                    onclick="hideInactivePostals();",
                    id="hideInactivePostalsButton",
                    style="display: none;",
                    icon="view--off",
                    buttontype="ghost",
                ),
                layout.button.Button(
                    _("Show inactive postals"),
                    onclick="showInactivePostals();",
                    id="showInactivePostalsButton",
                    icon="view",
                    buttontype="ghost",
                ),
            ),
            style="margin-top: 1.5rem;",
        ),
        R(
            C(
                modal_with_trigger(
                    modal_add_postal(),
                    layout.button.Button,
                    _("Add"),
                    buttontype="ghost",
                    icon="add",
                ),
            ),
        ),
        hg.SCRIPT(
            mark_safe(
                """
                function hideInactivePostals() {
                    for(i of $$('.inactive_postal')) {
                        $(i)._.style({display: "none"});
                    }
                    $$('#hideInactivePostalsButton')._.style({display: "none"})
                    $$('#showInactivePostalsButton')._.style({display: "block"})
                }
                function showInactivePostals() {
                    for(i of $$('.inactive_postal')) {
                        $(i)._.style({display: "block"});
                    }
                    $$('#hideInactivePostalsButton')._.style({display: "block"})
                    $$('#showInactivePostalsButton')._.style({display: "none"})
                }
                """
            )
        ),
    )


def modal_add_postal():
    return layout.modal.Modal.with_ajax_content(
        heading=_("Add Address"),
        url=hg.F(
            lambda c: reverse_model(
                models.Postal,
                "ajax_add",
                query={"asajax": True, "person": c["object"].pk},
            ),
        ),
        submitlabel=_("Save"),
    )



import htmlgenerator as hg
from bread import layout as layout
from bread.utils.urls import aslayout
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.forms import global_preference_form_builder

import basxconnect.core.layouts.settings_layout as settings_layout

R = layout.grid.Row
C = layout.grid.Col
F = layout.forms.FormField


@aslayout
def generalsettings(request):

    if request.method == "POST":
        form = global_preference_form_builder(
            preferences=["general__organizationname"]
        )(request.POST, request.FILES)
        if form.is_valid():
            form.update_preferences()
    else:
        form = global_preference_form_builder(
            preferences=["general__organizationname"]
        )()

    return layout.grid.Grid(
        R(C(hg.H3(_("Settings")))),
        R(C(hg.H4(_("General")))),
        R(C(hg.H5(_("Information about our organization")))),
        R(
            C(
                layout.forms.Form(
                    form,
                    hg.BaseElement(F("general__organizationname")),
                    layout.forms.helpers.Submit(),
                    style="max-width: 480px",
                )
            )
        ),
        gutter=False,
    )


@aslayout
def personsettings(request):
    return settings_layout.personsettings(request)


@aslayout
def relationshipssettings(request):
    return settings_layout.relationshipssettings(request)



from django.db.models.signals import post_save
from django.dispatch import receiver

from basxconnect.core.models import Person
from basxconnect.mailer_integration.abstract.mailer import MailerPerson
from basxconnect.mailer_integration.settings import MAILER


@receiver(post_save, sender=Person)
def update_subscription_active(sender, instance, created, **kwargs):

    if hasattr(instance, "core_email_list"):
        for email in instance.core_email_list.all():
            if hasattr(email, "subscription"):
                if (
                    instance.active
                    and not instance.deleted
                    and email.subscription.status == "archived"
                ):
                    email.subscription.status = (
                        email.subscription.status_before_archiving
                    )
                    email.subscription.status_before_archiving = None
                    MAILER.add_person(
                        MailerPerson.from_subscription(email.subscription)
                    )
                elif email.subscription.status != "archived":
                    email.subscription.status_before_archiving = (
                        email.subscription.status
                    )
                    email.subscription.status = "archived"
                    MAILER.delete_person(email.email)
                email.subscription.save()



from django import forms
from django.utils.translation import gettext_lazy as _
from dynamic_preferences.preferences import Section
from dynamic_preferences.registries import global_preferences_registry
from dynamic_preferences.types import (
    FilePreference,
    LongStringPreference,
    ModelChoicePreference,
    StringPreference,
)

from basxconnect.core.models import Term

general = Section("general", _("General"))
persons = Section("persons", _("Persons"))


@global_preferences_registry.register
class SenderLetterhead(LongStringPreference):
    section = general
    name = "sender_letterhead"
    default = ""
    verbose_name = _("Sender letterhead")


@global_preferences_registry.register
class Logo(FilePreference):
    section = general
    name = "logo"
    field_class = forms.ImageField
    default = ""
    verbose_name = _("Logo")

    field_kwargs = {
        "required": False,
    }


@global_preferences_registry.register
class OrganizationName(StringPreference):
    section = general
    name = "organizationname"
    default = "<Your Organization>"
    verbose_name = _("Organization Name")


@global_preferences_registry.register
class DefaultTypeNaturalPerson(ModelChoicePreference):
    section = persons
    name = "default_naturaltype"
    default = None
    queryset = Term.objects.filter(vocabulary__slug="naturaltype")
    verbose_name = _("Default type of natural persons")


@global_preferences_registry.register
class DefaultTypePersonAssociation(ModelChoicePreference):
    section = persons
    name = "default_associationtype"
    default = None
    queryset = Term.objects.filter(vocabulary__slug="associationtype")
    verbose_name = _("Default type of person associations")


@global_preferences_registry.register
class DefaultTypeLegalPerson(ModelChoicePreference):
    section = persons
    name = "default_legaltype"
    default = None
    queryset = Term.objects.filter(vocabulary__slug="legaltype")
    verbose_name = _("Default type of legal persons")



# Generated by Django 3.2.4 on 2021-08-23 13:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mailer_integration", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="interest",
            name="external_id",
            field=models.CharField(default=None, max_length=50, unique=True),
            preserve_default=False,
        ),
    ]



import traceback

import bread.layout.components.notification
import htmlgenerator as hg
from bread import layout, menu
from bread.layout.components.datatable import DataTableColumn
from bread.layout.components.forms import Form
from bread.utils import aslayout, reverse_model
from bread.utils.links import Link, ModelHref
from bread.views import AddView, EditView
from django import forms
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _

from basxconnect.mailer_integration import settings
from basxconnect.mailer_integration.abstract.mailer import MailerPerson
from basxconnect.mailer_integration.help import sync_help_modal
from basxconnect.mailer_integration.models import (
    SynchronizationPerson,
    SynchronizationResult,
)
from basxconnect.mailer_integration.synchronize import synchronize

C = bread.layout.grid.Col
R = bread.layout.grid.Row


@aslayout
def mailer_synchronization_view(request):
    if request.method == "POST":
        try:
            sync_result = synchronize(settings.MAILER)
            notification = bread.layout.components.notification.InlineNotification(
                _("Sychronization successful"),
                _(
                    "Synchronized with mailer segment containing %s contacts. %s new persons were added to BasxConnect."
                )
                % (
                    sync_result.total_synchronized_persons,
                    sync_result.persons.filter(
                        sync_status=SynchronizationPerson.NEW
                    ).count(),
                ),
                kind="success",
            )
        except Exception:
            notification = bread.layout.components.notification.InlineNotification(
                "Error",
                f"An error occured during synchronization. {traceback.format_exc()}",
                kind="error",
            )
    else:
        notification = None

    help_modal = sync_help_modal()
    return hg.BaseElement(
        Form(
            forms.Form(),
            bread.layout.grid.Grid(
                hg.H3(_("Synchronization of Email Subcriptions")),
                notification,
                gutter=False,
            ),
            help_modal,
            layout.forms.helpers.Submit(
                _("Download subscriptions"), style="display: inline-block;"
            ),
            layout.button.Button(
                _("Help"),
                buttontype="ghost",
                style="margin-left: 1rem",
                icon="help",
                **help_modal.openerattributes,
            ),
        ),
        display_previous_execution(request),
    )


def display_previous_execution(request):
    return R(
        C(
            layout.datatable.DataTable.from_queryset(
                SynchronizationResult.objects.order_by("-sync_completed_datetime"),
                columns=[
                    "total_synchronized_persons",
                    "sync_completed_datetime",
                    DataTableColumn(
                        _("Newly added to BasxConnect"),
                        display_new_persons(),
                    ),
                    DataTableColumn(
                        _("Subscription status changed"),
                        display_status_changed_persons(),
                    ),
                ],
                title=_("Previous Executions"),
                primary_button="",
                rowactions=[
                    Link(
                        href=ModelHref(
                            SynchronizationResult,
                            "delete",
                            kwargs={"pk": hg.C("row.pk")},
                            query={"next": request.get_full_path()},
                        ),
                        iconname="trash-can",
                        label=_("Delete"),
                    )
                ],
            ),
            width=16,
        )
    )


def display_new_persons():
    return hg.Iterator(
        hg.F(lambda c: c["row"].persons.filter(sync_status=SynchronizationPerson.NEW)),
        "person",
        hg.DIV(
            hg.format(
                "{} {} <{}>",
                hg.C("person.first_name"),
                hg.C("person.last_name"),
                hg.C("person.email"),
            )
        ),
    )


def display_status_changed_persons():
    return hg.Iterator(
        hg.F(
            lambda c: c["row"].persons.filter(
                sync_status=SynchronizationPerson.SUBSCRIPTION_STATUS_CHANGED
            )
        ),
        "person",
        hg.DIV(
            hg.format(
                "{} {} <{}>, {}: {}, {}: {}",
                hg.C("person.first_name"),
                hg.C("person.last_name"),
                hg.C("person.email"),
                _("Old"),
                hg.C("person.old_subscription_status"),
                _("New"),
                hg.C("person.new_subscription_status"),
            )
        ),
    )


tools_group = menu.Group(_("Tools"), iconname="tool", order=99)

menu.registeritem(
    menu.Item(
        Link(
            reverse_lazy(
                "basxconnect.mailer_integration.views.mailer_synchronization_view"
            ),
            settings.MAILER.name(),
            iconname="email",
        ),
        tools_group,
    )
)


class AddSubscriptionView(AddView):
    fields = ["interests", "language", "email", "status"]

    def get_success_url(self):
        return reverse_model(
            self.object.email.person, "read", kwargs={"pk": self.object.email.person.pk}
        )

    def post(self, request, *args, **kwargs):
        response = super().post(request, *args, **kwargs)
        settings.MAILER.add_person(MailerPerson.from_subscription(self.object))
        return response


class EditSubscriptionView(EditView):
    fields = ["interests", "language"]

    def get_success_url(self):
        return reverse_model(
            self.object.email.person, "read", kwargs={"pk": self.object.email.person.pk}
        )

    def post(self, request, *args, **kwargs):
        result = super().post(request, *args, **kwargs)
        # TODO: https://github.com/basxsoftwareassociation/basxconnect/issues/140
        settings.MAILER.put_person(MailerPerson.from_subscription(self.object))
        return result



from bread import layout
from bread.utils import Link, ModelHref, pretty_modelname
from bread.views import BrowseView
from django.utils.translation import gettext_lazy as _

from basxconnect.core.models import Term, Vocabulary


class TermsBrowseView(BrowseView):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get(self, *args, **kwargs):
        vocabulary = Vocabulary.objects.filter(
            slug=self.request.GET.get("vocabulary_slug")
        ).first()
        if vocabulary:
            self.primary_button = layout.button.Button.from_link(
                Link(
                    href=ModelHref(
                        Term,
                        "add",
                        query={"vocabulary": vocabulary.id},
                        return_to_current=True,
                    ),
                    label=_("Add %s") % pretty_modelname(Term),
                ),
                icon=layout.icon.Icon("add", size=20),
            )
        return super().get(*args, **kwargs)

    def get_queryset(self):
        qs = super().get_queryset()
        vocabulary_slug = self.request.GET.get("vocabulary_slug")
        if vocabulary_slug:
            qs = qs.filter(vocabulary__slug=vocabulary_slug)
        return qs



# Generated by Django 3.2.4 on 2021-06-24 13:54

import djmoney.models.fields
from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("contributions", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="contribution",
            name="currency",
            field=djmoney.models.fields.CurrencyField(
                choices=[
                    ("XUA", "ADB Unit of Account"),
                    ("AFN", "Afghan Afghani"),
                    ("AFA", "Afghan Afghani (1927–2002)"),
                    ("ALL", "Albanian Lek"),
                    ("ALK", "Albanian Lek (1946–1965)"),
                    ("DZD", "Algerian Dinar"),
                    ("ADP", "Andorran Peseta"),
                    ("AOA", "Angolan Kwanza"),
                    ("AOK", "Angolan Kwanza (1977–1991)"),
                    ("AON", "Angolan New Kwanza (1990–2000)"),
                    ("AOR", "Angolan Readjusted Kwanza (1995–1999)"),
                    ("ARA", "Argentine Austral"),
                    ("ARS", "Argentine Peso"),
                    ("ARM", "Argentine Peso (1881–1970)"),
                    ("ARP", "Argentine Peso (1983–1985)"),
                    ("ARL", "Argentine Peso Ley (1970–1983)"),
                    ("AMD", "Armenian Dram"),
                    ("AWG", "Aruban Florin"),
                    ("AUD", "Australian Dollar"),
                    ("ATS", "Austrian Schilling"),
                    ("AZN", "Azerbaijani Manat"),
                    ("AZM", "Azerbaijani Manat (1993–2006)"),
                    ("BSD", "Bahamian Dollar"),
                    ("BHD", "Bahraini Dinar"),
                    ("BDT", "Bangladeshi Taka"),
                    ("BBD", "Barbadian Dollar"),
                    ("BYN", "Belarusian Ruble"),
                    ("BYB", "Belarusian Ruble (1994–1999)"),
                    ("BYR", "Belarusian Ruble (2000–2016)"),
                    ("BEF", "Belgian Franc"),
                    ("BEC", "Belgian Franc (convertible)"),
                    ("BEL", "Belgian Franc (financial)"),
                    ("BZD", "Belize Dollar"),
                    ("BMD", "Bermudan Dollar"),
                    ("BTN", "Bhutanese Ngultrum"),
                    ("BOB", "Bolivian Boliviano"),
                    ("BOL", "Bolivian Boliviano (1863–1963)"),
                    ("BOV", "Bolivian Mvdol"),
                    ("BOP", "Bolivian Peso"),
                    ("BAM", "Bosnia-Herzegovina Convertible Mark"),
                    ("BAD", "Bosnia-Herzegovina Dinar (1992–1994)"),
                    ("BAN", "Bosnia-Herzegovina New Dinar (1994–1997)"),
                    ("BWP", "Botswanan Pula"),
                    ("BRC", "Brazilian Cruzado (1986–1989)"),
                    ("BRZ", "Brazilian Cruzeiro (1942–1967)"),
                    ("BRE", "Brazilian Cruzeiro (1990–1993)"),
                    ("BRR", "Brazilian Cruzeiro (1993–1994)"),
                    ("BRN", "Brazilian New Cruzado (1989–1990)"),
                    ("BRB", "Brazilian New Cruzeiro (1967–1986)"),
                    ("BRL", "Brazilian Real"),
                    ("GBP", "British Pound"),
                    ("BND", "Brunei Dollar"),
                    ("BGL", "Bulgarian Hard Lev"),
                    ("BGN", "Bulgarian Lev"),
                    ("BGO", "Bulgarian Lev (1879–1952)"),
                    ("BGM", "Bulgarian Socialist Lev"),
                    ("BUK", "Burmese Kyat"),
                    ("BIF", "Burundian Franc"),
                    ("XPF", "CFP Franc"),
                    ("KHR", "Cambodian Riel"),
                    ("CAD", "Canadian Dollar"),
                    ("CVE", "Cape Verdean Escudo"),
                    ("KYD", "Cayman Islands Dollar"),
                    ("XAF", "Central African CFA Franc"),
                    ("CLE", "Chilean Escudo"),
                    ("CLP", "Chilean Peso"),
                    ("CLF", "Chilean Unit of Account (UF)"),
                    ("CNX", "Chinese People’s Bank Dollar"),
                    ("CNY", "Chinese Yuan"),
                    ("CNH", "Chinese Yuan (offshore)"),
                    ("COP", "Colombian Peso"),
                    ("COU", "Colombian Real Value Unit"),
                    ("KMF", "Comorian Franc"),
                    ("CDF", "Congolese Franc"),
                    ("CRC", "Costa Rican Colón"),
                    ("HRD", "Croatian Dinar"),
                    ("HRK", "Croatian Kuna"),
                    ("CUC", "Cuban Convertible Peso"),
                    ("CUP", "Cuban Peso"),
                    ("CYP", "Cypriot Pound"),
                    ("CZK", "Czech Koruna"),
                    ("CSK", "Czechoslovak Hard Koruna"),
                    ("DKK", "Danish Krone"),
                    ("DJF", "Djiboutian Franc"),
                    ("DOP", "Dominican Peso"),
                    ("NLG", "Dutch Guilder"),
                    ("XCD", "East Caribbean Dollar"),
                    ("DDM", "East German Mark"),
                    ("ECS", "Ecuadorian Sucre"),
                    ("ECV", "Ecuadorian Unit of Constant Value"),
                    ("EGP", "Egyptian Pound"),
                    ("GQE", "Equatorial Guinean Ekwele"),
                    ("ERN", "Eritrean Nakfa"),
                    ("EEK", "Estonian Kroon"),
                    ("ETB", "Ethiopian Birr"),
                    ("EUR", "Euro"),
                    ("XBA", "European Composite Unit"),
                    ("XEU", "European Currency Unit"),
                    ("XBB", "European Monetary Unit"),
                    ("XBC", "European Unit of Account (XBC)"),
                    ("XBD", "European Unit of Account (XBD)"),
                    ("FKP", "Falkland Islands Pound"),
                    ("FJD", "Fijian Dollar"),
                    ("FIM", "Finnish Markka"),
                    ("FRF", "French Franc"),
                    ("XFO", "French Gold Franc"),
                    ("XFU", "French UIC-Franc"),
                    ("GMD", "Gambian Dalasi"),
                    ("GEK", "Georgian Kupon Larit"),
                    ("GEL", "Georgian Lari"),
                    ("DEM", "German Mark"),
                    ("GHS", "Ghanaian Cedi"),
                    ("GHC", "Ghanaian Cedi (1979–2007)"),
                    ("GIP", "Gibraltar Pound"),
                    ("XAU", "Gold"),
                    ("GRD", "Greek Drachma"),
                    ("GTQ", "Guatemalan Quetzal"),
                    ("GWP", "Guinea-Bissau Peso"),
                    ("GNF", "Guinean Franc"),
                    ("GNS", "Guinean Syli"),
                    ("GYD", "Guyanaese Dollar"),
                    ("HTG", "Haitian Gourde"),
                    ("HNL", "Honduran Lempira"),
                    ("HKD", "Hong Kong Dollar"),
                    ("HUF", "Hungarian Forint"),
                    ("IMP", "IMP"),
                    ("ISK", "Icelandic Króna"),
                    ("ISJ", "Icelandic Króna (1918–1981)"),
                    ("INR", "Indian Rupee"),
                    ("IDR", "Indonesian Rupiah"),
                    ("IRR", "Iranian Rial"),
                    ("IQD", "Iraqi Dinar"),
                    ("IEP", "Irish Pound"),
                    ("ILS", "Israeli New Shekel"),
                    ("ILP", "Israeli Pound"),
                    ("ILR", "Israeli Shekel (1980–1985)"),
                    ("ITL", "Italian Lira"),
                    ("JMD", "Jamaican Dollar"),
                    ("JPY", "Japanese Yen"),
                    ("JOD", "Jordanian Dinar"),
                    ("KZT", "Kazakhstani Tenge"),
                    ("KES", "Kenyan Shilling"),
                    ("KWD", "Kuwaiti Dinar"),
                    ("KGS", "Kyrgystani Som"),
                    ("LAK", "Laotian Kip"),
                    ("LVL", "Latvian Lats"),
                    ("LVR", "Latvian Ruble"),
                    ("LBP", "Lebanese Pound"),
                    ("LSL", "Lesotho Loti"),
                    ("LRD", "Liberian Dollar"),
                    ("LYD", "Libyan Dinar"),
                    ("LTL", "Lithuanian Litas"),
                    ("LTT", "Lithuanian Talonas"),
                    ("LUL", "Luxembourg Financial Franc"),
                    ("LUC", "Luxembourgian Convertible Franc"),
                    ("LUF", "Luxembourgian Franc"),
                    ("MOP", "Macanese Pataca"),
                    ("MKD", "Macedonian Denar"),
                    ("MKN", "Macedonian Denar (1992–1993)"),
                    ("MGA", "Malagasy Ariary"),
                    ("MGF", "Malagasy Franc"),
                    ("MWK", "Malawian Kwacha"),
                    ("MYR", "Malaysian Ringgit"),
                    ("MVR", "Maldivian Rufiyaa"),
                    ("MVP", "Maldivian Rupee (1947–1981)"),
                    ("MLF", "Malian Franc"),
                    ("MTL", "Maltese Lira"),
                    ("MTP", "Maltese Pound"),
                    ("MRU", "Mauritanian Ouguiya"),
                    ("MRO", "Mauritanian Ouguiya (1973–2017)"),
                    ("MUR", "Mauritian Rupee"),
                    ("MXV", "Mexican Investment Unit"),
                    ("MXN", "Mexican Peso"),
                    ("MXP", "Mexican Silver Peso (1861–1992)"),
                    ("MDC", "Moldovan Cupon"),
                    ("MDL", "Moldovan Leu"),
                    ("MCF", "Monegasque Franc"),
                    ("MNT", "Mongolian Tugrik"),
                    ("MAD", "Moroccan Dirham"),
                    ("MAF", "Moroccan Franc"),
                    ("MZE", "Mozambican Escudo"),
                    ("MZN", "Mozambican Metical"),
                    ("MZM", "Mozambican Metical (1980–2006)"),
                    ("MMK", "Myanmar Kyat"),
                    ("NAD", "Namibian Dollar"),
                    ("NPR", "Nepalese Rupee"),
                    ("ANG", "Netherlands Antillean Guilder"),
                    ("TWD", "New Taiwan Dollar"),
                    ("NZD", "New Zealand Dollar"),
                    ("NIO", "Nicaraguan Córdoba"),
                    ("NIC", "Nicaraguan Córdoba (1988–1991)"),
                    ("NGN", "Nigerian Naira"),
                    ("KPW", "North Korean Won"),
                    ("NOK", "Norwegian Krone"),
                    ("OMR", "Omani Rial"),
                    ("PKR", "Pakistani Rupee"),
                    ("XPD", "Palladium"),
                    ("PAB", "Panamanian Balboa"),
                    ("PGK", "Papua New Guinean Kina"),
                    ("PYG", "Paraguayan Guarani"),
                    ("PEI", "Peruvian Inti"),
                    ("PEN", "Peruvian Sol"),
                    ("PES", "Peruvian Sol (1863–1965)"),
                    ("PHP", "Philippine Piso"),
                    ("XPT", "Platinum"),
                    ("PLN", "Polish Zloty"),
                    ("PLZ", "Polish Zloty (1950–1995)"),
                    ("PTE", "Portuguese Escudo"),
                    ("GWE", "Portuguese Guinea Escudo"),
                    ("QAR", "Qatari Rial"),
                    ("XRE", "RINET Funds"),
                    ("RHD", "Rhodesian Dollar"),
                    ("RON", "Romanian Leu"),
                    ("ROL", "Romanian Leu (1952–2006)"),
                    ("RUB", "Russian Ruble"),
                    ("RUR", "Russian Ruble (1991–1998)"),
                    ("RWF", "Rwandan Franc"),
                    ("SVC", "Salvadoran Colón"),
                    ("WST", "Samoan Tala"),
                    ("SAR", "Saudi Riyal"),
                    ("RSD", "Serbian Dinar"),
                    ("CSD", "Serbian Dinar (2002–2006)"),
                    ("SCR", "Seychellois Rupee"),
                    ("SLL", "Sierra Leonean Leone"),
                    ("XAG", "Silver"),
                    ("SGD", "Singapore Dollar"),
                    ("SKK", "Slovak Koruna"),
                    ("SIT", "Slovenian Tolar"),
                    ("SBD", "Solomon Islands Dollar"),
                    ("SOS", "Somali Shilling"),
                    ("ZAR", "South African Rand"),
                    ("ZAL", "South African Rand (financial)"),
                    ("KRH", "South Korean Hwan (1953–1962)"),
                    ("KRW", "South Korean Won"),
                    ("KRO", "South Korean Won (1945–1953)"),
                    ("SSP", "South Sudanese Pound"),
                    ("SUR", "Soviet Rouble"),
                    ("ESP", "Spanish Peseta"),
                    ("ESA", "Spanish Peseta (A account)"),
                    ("ESB", "Spanish Peseta (convertible account)"),
                    ("XDR", "Special Drawing Rights"),
                    ("LKR", "Sri Lankan Rupee"),
                    ("SHP", "St. Helena Pound"),
                    ("XSU", "Sucre"),
                    ("SDD", "Sudanese Dinar (1992–2007)"),
                    ("SDG", "Sudanese Pound"),
                    ("SDP", "Sudanese Pound (1957–1998)"),
                    ("SRD", "Surinamese Dollar"),
                    ("SRG", "Surinamese Guilder"),
                    ("SZL", "Swazi Lilangeni"),
                    ("SEK", "Swedish Krona"),
                    ("CHF", "Swiss Franc"),
                    ("SYP", "Syrian Pound"),
                    ("STN", "São Tomé & Príncipe Dobra"),
                    ("STD", "São Tomé & Príncipe Dobra (1977–2017)"),
                    ("TVD", "TVD"),
                    ("TJR", "Tajikistani Ruble"),
                    ("TJS", "Tajikistani Somoni"),
                    ("TZS", "Tanzanian Shilling"),
                    ("XTS", "Testing Currency Code"),
                    ("THB", "Thai Baht"),
                    (
                        "XXX",
                        "The codes assigned for transactions where no currency is involved",
                    ),
                    ("TPE", "Timorese Escudo"),
                    ("TOP", "Tongan Paʻanga"),
                    ("TTD", "Trinidad & Tobago Dollar"),
                    ("TND", "Tunisian Dinar"),
                    ("TRY", "Turkish Lira"),
                    ("TRL", "Turkish Lira (1922–2005)"),
                    ("TMT", "Turkmenistani Manat"),
                    ("TMM", "Turkmenistani Manat (1993–2009)"),
                    ("USD", "US Dollar"),
                    ("USN", "US Dollar (Next day)"),
                    ("USS", "US Dollar (Same day)"),
                    ("UGX", "Ugandan Shilling"),
                    ("UGS", "Ugandan Shilling (1966–1987)"),
                    ("UAH", "Ukrainian Hryvnia"),
                    ("UAK", "Ukrainian Karbovanets"),
                    ("AED", "United Arab Emirates Dirham"),
                    ("UYW", "Uruguayan Nominal Wage Index Unit"),
                    ("UYU", "Uruguayan Peso"),
                    ("UYP", "Uruguayan Peso (1975–1993)"),
                    ("UYI", "Uruguayan Peso (Indexed Units)"),
                    ("UZS", "Uzbekistani Som"),
                    ("VUV", "Vanuatu Vatu"),
                    ("VES", "Venezuelan Bolívar"),
                    ("VEB", "Venezuelan Bolívar (1871–2008)"),
                    ("VEF", "Venezuelan Bolívar (2008–2018)"),
                    ("VND", "Vietnamese Dong"),
                    ("VNN", "Vietnamese Dong (1978–1985)"),
                    ("CHE", "WIR Euro"),
                    ("CHW", "WIR Franc"),
                    ("XOF", "West African CFA Franc"),
                    ("YDD", "Yemeni Dinar"),
                    ("YER", "Yemeni Rial"),
                    ("YUN", "Yugoslavian Convertible Dinar (1990–1992)"),
                    ("YUD", "Yugoslavian Hard Dinar (1966–1990)"),
                    ("YUM", "Yugoslavian New Dinar (1994–2002)"),
                    ("YUR", "Yugoslavian Reformed Dinar (1992–1993)"),
                    ("ZWN", "ZWN"),
                    ("ZRN", "Zairean New Zaire (1993–1998)"),
                    ("ZRZ", "Zairean Zaire (1971–1993)"),
                    ("ZMW", "Zambian Kwacha"),
                    ("ZMK", "Zambian Kwacha (1968–2012)"),
                    ("ZWD", "Zimbabwean Dollar (1980–2008)"),
                    ("ZWR", "Zimbabwean Dollar (2008)"),
                    ("ZWL", "Zimbabwean Dollar (2009)"),
                ],
                default="XYZ",
                max_length=3,
            ),
        ),
    ]



from bread import layout
from bread.layout.components.icon import Icon
from django.utils.translation import gettext_lazy as _

from basxconnect.core import models
from basxconnect.core.layouts.editperson.common.utils import (
    grid_inside_tab,
    tile_col_edit_modal,
    tiling_col,
)

R = layout.grid.Row
C = layout.grid.Col


def mailings_tab(request):
    from django.apps import apps

    if apps.is_installed("basxconnect.mailer_integration"):
        from basxconnect.mailer_integration.layouts import mailer_integration_tile

        mailer_tile = mailer_integration_tile(request)
    else:
        mailer_tile = tiling_col()

    return layout.tabs.Tab(
        _("Mailings"),
        grid_inside_tab(
            R(
                tile_col_edit_modal(
                    _("Settings"),
                    models.NaturalPerson,
                    "ajax_edit_mailings",
                    Icon("settings--adjust"),
                    [
                        "preferred_language",
                        "type",
                        "salutation_letter",
                        "gender",
                        "form_of_address",
                    ],
                ),
                mailer_tile,
            ),
        ),
    )



# Generated by Django 3.1.5 on 2021-03-11 11:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0003_auto_20210309_2221"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalnaturalperson",
            name="form_of_address",
        ),
        migrations.RemoveField(
            model_name="naturalperson",
            name="form_of_address",
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="decease_date",
            field=models.DateField(blank=True, null=True, verbose_name="Deceased Date"),
        ),
        migrations.AlterField(
            model_name="historicalnaturalperson",
            name="deceased",
            field=models.BooleanField(default=False, verbose_name="Deceased"),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="decease_date",
            field=models.DateField(blank=True, null=True, verbose_name="Deceased Date"),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="deceased",
            field=models.BooleanField(default=False, verbose_name="Deceased"),
        ),
        migrations.AlterField(
            model_name="naturalperson",
            name="salutation",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"category__slug": "salutation"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="core.term",
            ),
        ),
    ]



from bread.contrib.languages.fields import LanguageField
from django.db import models
from django.utils.translation import gettext_lazy as _

from basxconnect.core.models import Email


class Interest(models.Model):
    name = models.CharField(max_length=100, unique=True)
    external_id = models.CharField(max_length=50)

    def __str__(self):
        return self.name


def languages_choices(field, request, instance):
    from basxconnect.core import settings

    return settings.PREFERRED_LANGUAGES


class SynchronizationResult(models.Model):
    total_synchronized_persons = models.IntegerField(
        null=True, verbose_name=_("Number of persons in the mailer segment")
    )
    sync_completed_datetime = models.DateTimeField(
        null=True, verbose_name=_("Date and time")
    )


class Subscription(models.Model):
    email = models.OneToOneField(Email, on_delete=models.CASCADE)
    status = models.CharField(
        max_length=50,
        choices=[
            ("subscribed", "subscribed"),
            ("unsubscribed", "unsubscribed"),
            ("non-subscribed", "non-subscribed"),
            ("cleaned", "cleaned"),
            ("archived", "archived"),
        ],
    )
    status_before_archiving = models.CharField(
        max_length=50,
        choices=[
            ("subscribed", "subscribed"),
            ("unsubscribed", "unsubscribed"),
            ("non-subscribed", "non-subscribed"),
            ("cleaned", "cleaned"),
        ],
        null=True,
    )

    interests = models.ManyToManyField(Interest, blank=True)
    interests.verbose_name = _("Mailing Interests")

    language = LanguageField(_("Language"), blank=True)
    language.lazy_choices = languages_choices

    latest_sync = models.ForeignKey(
        SynchronizationResult,
        related_name="synchronized_subscription",
        on_delete=models.SET_NULL,
        null=True,
    )


class SynchronizationPerson(models.Model):
    sync_result = models.ForeignKey(
        SynchronizationResult, on_delete=models.CASCADE, related_name="persons"
    )
    email = models.CharField(max_length=100)
    first_name = models.CharField(_("First Name"), max_length=255, blank=True)
    last_name = models.CharField(_("Last Name"), max_length=255, blank=True)
    old_subscription_status = models.CharField(
        _("Old subscription status"), max_length=255, blank=True
    )
    new_subscription_status = models.CharField(
        _("New subscription status"), max_length=255, blank=True
    )

    NEW = "new"
    SKIPPED = "import_error"
    PREVIOUSLY_SYNCED = "synced_previously"
    SUBSCRIPTION_STATUS_CHANGED = "subscription_status_changed"
    SYNC_STATUS_CHOICES = [
        (NEW, _("Newly added to BasxConnect")),
        (SKIPPED, _("Not added to BasxConnect")),
        (PREVIOUSLY_SYNCED, _("Synchronized previously but not this time")),
        (SUBSCRIPTION_STATUS_CHANGED, _("Subscription status changed")),
    ]
    sync_status = models.CharField(
        _("Synchronization Status"),
        choices=SYNC_STATUS_CHOICES,
        max_length=255,
        blank=True,
    )

    def __str__(self):
        return self.email



import htmlgenerator as hg
from bread import layout, menu, views
from bread.formatters import format_value
from bread.utils.links import Link, ModelHref
from bread.utils.urls import autopath, default_model_paths, model_urlname, reverse
from django.utils.translation import gettext_lazy as _
from django.views.generic import RedirectView

from . import models
from .wizards.contributionsimport import ContributionsImportWizard

urlpatterns = [
    autopath(
        RedirectView.as_view(url=reverse("importcontributions", kwargs={"step": "1"})),
        model_urlname(models.ContributionImport, "add"),
    ),
    *default_model_paths(
        models.ContributionImport,
        browseview=views.BrowseView._with(
            columns=(
                layout.datatable.DataTableColumn(
                    _("Import date"), layout.FC("row.date.date"), None
                ),
                layout.datatable.DataTableColumn(
                    _("Importfile"),
                    hg.BaseElement(
                        layout.ObjectFieldValue(
                            "importfile", "row", formatter=format_value
                        ),
                    ),
                    None,
                ),
                "user",
                "bookingrange",
                "numberofbookings",
                "totalamount",
            ),
        ),
    ),
    *default_model_paths(models.Contribution),
    autopath(
        ContributionsImportWizard.as_view(url_name="importcontributions"),
        "importcontributions",
    ),
]

importgroup = menu.Group(_("Imports"), iconname="document--import")

menu.registeritem(
    menu.Item(
        Link(
            ModelHref(models.ContributionImport, "browse"),
            _("Contributions"),
            iconname="money",
        ),
        importgroup,
    )
)



import django_countries
from django.utils import timezone
from dynamic_preferences.registries import global_preferences_registry

from basxconnect.core import models
from basxconnect.core.models import Email
from basxconnect.mailer_integration.abstract.mailer import AbstractMailer, MailerPerson
from basxconnect.mailer_integration.models import (
    Interest,
    Subscription,
    SynchronizationPerson,
    SynchronizationResult,
)


def synchronize(mailer: AbstractMailer) -> SynchronizationResult:
    synchronize_interests(mailer)

    person_count = mailer.get_person_count()
    sync_result = SynchronizationResult.objects.create()
    sync_result.total_synchronized_persons = person_count
    batch_size = 1000
    for batch_number in range(((person_count - 1) // batch_size) + 1):
        synchronize_batch(batch_size, batch_number * batch_size, mailer, sync_result)
    sync_result.sync_completed_datetime = timezone.now()
    sync_result.save()

    # delete all subscriptions which were not synchronized
    # and were not deactivated already using the UI (those we want to be able to reactivate, so we have to keep them)
    Subscription.objects.exclude(latest_sync=sync_result).exclude(
        status="archived"
    ).delete()

    return sync_result


def synchronize_batch(count, offset, mailer, sync_result):
    mailer_persons = mailer.get_persons(count, offset)
    datasource_tag = _get_or_create_tag(mailer.tag())
    for mailer_person in mailer_persons:
        matching_email_addresses = list(
            models.Email.objects.filter(
                email=mailer_person.email,
            ).all()
        )
        if len(matching_email_addresses) == 0:
            if not is_valid_new_person(mailer_person):
                _save_sync_person(
                    mailer_person, sync_result, SynchronizationPerson.SKIPPED
                )
            else:
                created_person = _save_person(datasource_tag, mailer_person)
                _save_subscription(
                    created_person.primary_email_address, mailer_person, sync_result
                )
                _save_sync_person(mailer_person, sync_result, SynchronizationPerson.NEW)
        else:
            # if the downloaded email address already exists in our system, update the mailing preference for this email
            # address, without creating a new person in the database
            for email in matching_email_addresses:
                _save_subscription(email, mailer_person, sync_result)


def synchronize_interests(datasource):
    old_interests = Interest.objects.all()
    downloaded_interests = datasource.get_interests()
    new_interests_ids = []
    for interest in downloaded_interests:
        interest_from_db, _ = Interest.objects.get_or_create(
            external_id=interest.id, name=interest.name
        )
        new_interests_ids.append(interest_from_db.id)
    for interest in old_interests:
        if interest.id not in new_interests_ids:
            interest.delete()


def _get_or_create_tag(tag: str) -> models.Term:
    tags_vocabulary = models.Vocabulary.objects.get(slug="tag")
    tag, _ = models.Term.objects.get_or_create(term=tag, vocabulary=tags_vocabulary)
    return tag


def is_valid_new_person(person: MailerPerson):
    return django_countries.Countries().countries.get(
        person.country
    ) and person.status in ["subscribed"]


def _save_sync_person(
    mailer_person, sync_result, syn_status, old_subscription_status=""
):
    SynchronizationPerson.objects.create(
        sync_result=sync_result,
        email=mailer_person.email,
        first_name=mailer_person.first_name,
        last_name=mailer_person.last_name,
        sync_status=syn_status,
        new_subscription_status=mailer_person.status,
        old_subscription_status=old_subscription_status,
    )


def _save_person(datasource_tag: models.Term, mailer_person: MailerPerson):
    person = models.NaturalPerson.objects.create(
        first_name=mailer_person.first_name,
        name=mailer_person.display_name,
        last_name=mailer_person.last_name,
    )
    if mailer_person.persontype:
        persontype = models.Term.objects.filter(slug=MailerPerson.persontype).first()
        if persontype:
            person._type = persontype
    person.tags.add(datasource_tag)
    person.save()
    email = models.Email.objects.create(email=mailer_person.email, person=person)
    person.primary_email_address = email
    person.save()
    _save_postal_address(person, mailer_person)
    return person


def _save_postal_address(person: models.Person, mailer_person: MailerPerson):
    address = models.Postal(
        person=person,
        country=mailer_person.country,
        address=mailer_person.address,
        postcode=mailer_person.postcode,
        city=mailer_person.city,
    )
    address.save()
    person.primary_postal_address = address
    person.save()


def _save_subscription(
    email: models.Email, mailer_person: MailerPerson, sync_result: SynchronizationResult
):
    subscription, _ = Subscription.objects.get_or_create(email=email)
    old_subscription_status = subscription.status or ""
    subscription.status = mailer_person.status
    subscription.language = mailer_person.language
    subscription.interests.clear()
    for interest_id in mailer_person.interests_ids:
        interest = Interest.objects.get(external_id=interest_id)
        subscription.interests.add(interest)
    subscription.latest_sync = sync_result
    subscription.save()
    if old_subscription_status != subscription.status:
        _save_sync_person(
            mailer_person,
            sync_result,
            SynchronizationPerson.SUBSCRIPTION_STATUS_CHANGED,
            old_subscription_status=old_subscription_status,
        )
    if global_preferences_registry.manager()["mailchimp__synchronize_language"]:
        person = Email.objects.get(email=email).person
        person.preferred_language = mailer_person.language
        person.save()



import htmlgenerator as hg
from bread import layout as layout
from bread.layout.components.datatable import DataTableColumn
from bread.utils import get_concrete_instance
from bread.utils.links import Link
from bread.utils.urls import reverse, reverse_model
from bread.views import BrowseView, BulkAction
from bread.views.browse import delete as breaddelete
from bread.views.browse import export as breadexport
from bread.views.browse import restore as breadrestore
from django import forms
from django.db.models import Q
from django.http import HttpResponseRedirect
from django.utils.html import mark_safe
from django.utils.translation import gettext_lazy as _
from django.utils.translation import pgettext_lazy

from basxconnect.core import models, settings


def bulkdelete(request, qs):
    breaddelete(request, qs, softdeletefield="deleted")
    for person in qs:
        person.active = False
        person.save()


def bulkrestore(request, qs):
    breadrestore(request, qs, softdeletefield="deleted")
    for person in qs:
        person.active = True
        person.save()


def bulkaddtag(request, qs):
    return _redirect_to_tag_operation(qs, "add")


def bulkremovetag(request, qs):
    return _redirect_to_tag_operation(qs, "remove")


def _redirect_to_tag_operation(qs, operation):
    return HttpResponseRedirect(
        reverse_model(
            models.Person,
            "bulk-tag-operation",
            query={
                "operation": operation,
                "persons": [str(person.id) for person in qs],
            },
        )
    )


def export(request, queryset):
    # Fields which are filtered should also be displayed in columns
    form = PersonBrowseView.FilterForm({"status": ["active"], **request.GET})
    columns = list(PersonBrowseView.columns)
    if form.is_valid():

        # only the tags selected in the filter should be visible in the export
        tags = models.Term.objects.all()
        if form.cleaned_data.get("tags"):
            tags = form.cleaned_data.get("tags")

        def render_matching_tags(context):
            return ", ".join(str(i) for i in set(tags) & set(context["row"].tags.all()))

        columns.append(
            DataTableColumn(
                layout.ObjectFieldLabel("tags", models.Person),
                hg.F(render_matching_tags),
            )
        )
        if form.cleaned_data.get("preferred_language"):
            columns.append("preferred_language")

    def get_from_concret_object(field):
        return hg.F(lambda c: getattr(get_concrete_instance(c["row"]), field, ""))

    # insert last_name and first_name
    try:
        name_field = [getattr(i, "sortingname", i) for i in columns].index(
            "default_sorting_name"
        )
        columns.insert(
            name_field + 1,
            DataTableColumn(
                layout.ObjectFieldLabel("first_name", models.NaturalPerson),
                get_from_concret_object("first_name"),
            ),
        )
    except ValueError:
        pass

    return breadexport(queryset=queryset, columns=columns)


class PersonBrowseView(BrowseView):
    columns = [
        DataTableColumn(
            layout.ObjectFieldLabel("personnumber", models.Person),
            hg.DIV(
                hg.C("row.personnumber"),
                style=hg.If(hg.C("row.deleted"), "text-decoration:line-through"),
            ),
            "personnumber__int",
        ),
        DataTableColumn(
            hg.DIV(
                layout.tooltip.IconTooltip(
                    hg.UL(
                        hg.LI(hg.SPAN("●", style="color: green"), " ", _("Active")),
                        hg.LI(hg.SPAN("●", style="color: red"), " ", _("Inactive")),
                    ),
                    position="top",
                )
            ),
            hg.DIV(
                "●",
                style=hg.format("color: {}", hg.If(hg.C("row.active"), "green", "red")),
            ),
            "active",
        ),
        DataTableColumn(
            layout.ObjectFieldLabel("_type", models.Person), hg.C("row._type"), "_type"
        ),
        DataTableColumn(
            _("Name"),
            hg.DIV(
                hg.If(
                    hg.F(lambda context: type(context["row"]) == models.NaturalPerson),
                    hg.C("row.last_name"),
                    hg.C("row.name"),
                ),
                style=hg.If(hg.C("row.deleted"), "text-decoration:line-through"),
            ),
            "default_sorting_name",
        ),
        DataTableColumn(
            layout.ObjectFieldLabel("first_name", models.NaturalPerson),
            hg.DIV(
                hg.If(
                    hg.F(lambda context: type(context["row"]) == models.NaturalPerson),
                    hg.C("row.first_name"),
                    "",
                ),
                style=hg.If(hg.C("row.deleted"), "text-decoration:line-through"),
            ),
            "naturalperson__first_name",
        ),
        "primary_postal_address.address",
        "primary_postal_address.postcode",
        "primary_postal_address.city",
        "primary_postal_address.country",
        DataTableColumn(
            _("Email"),
            hg.C(
                "row.primary_email_address.asbutton",
            ),
            "primary_email_address__email",
            False,
        ),
        DataTableColumn(
            layout.ObjectFieldLabel("tags", models.Person),
            hg.UL(
                hg.Iterator(hg.C("row.tags.all"), "tag", layout.tag.Tag(hg.C("tag")))
            ),
        ),
    ]
    bulkactions = (
        BulkAction(
            "add-tag",
            label=_("Add tag"),
            iconname="add",
            action=bulkaddtag,
        ),
        BulkAction(
            "remove-tag",
            label=_("Remove tag"),
            iconname="subtract",
            action=bulkremovetag,
        ),
        BulkAction(
            "delete",
            label=_("Delete"),
            iconname="trash-can",
            action=bulkdelete,
        ),
        BulkAction(
            "restore",
            label=_("Restore"),
            iconname="restart",
            action=bulkrestore,
        ),
        BulkAction(
            "excel",
            label=_("Excel"),
            iconname="download",
            action=export,
        ),
    )
    search_backend = layout.search.SearchBackendConfig(
        url=reverse("basxconnect.core.views.person.search_person_view.searchperson")
    )
    rowclickaction = BrowseView.gen_rowclickaction("read")
    viewstate_sessionkey = "personbrowseview"

    class FilterForm(forms.Form):
        naturalperson = forms.BooleanField(required=False, label=_("Natural Person"))
        legalperson = forms.BooleanField(required=False, label=_("Legal Person"))
        personassociation = forms.BooleanField(
            required=False, label=_("Person Association")
        )
        naturalperson_subtypes = forms.ModelMultipleChoiceField(
            queryset=models.Term.objects.filter(vocabulary__slug="naturaltype"),
            widget=forms.CheckboxSelectMultiple,
            required=False,
            label="",
        )
        legalperson_subtypes = forms.ModelMultipleChoiceField(
            queryset=models.Term.objects.filter(vocabulary__slug="legaltype"),
            widget=forms.CheckboxSelectMultiple,
            required=False,
            label="",
        )
        personassociation_subtypes = forms.ModelMultipleChoiceField(
            queryset=models.Term.objects.filter(vocabulary__slug="associationtype"),
            widget=forms.CheckboxSelectMultiple,
            required=False,
            label="",
        )
        tags = forms.ModelMultipleChoiceField(
            queryset=models.Term.objects.filter(vocabulary__slug="tag"),
            widget=forms.CheckboxSelectMultiple,
            required=False,
        )
        preferred_language = forms.MultipleChoiceField(
            choices=settings.PREFERRED_LANGUAGES,
            widget=forms.CheckboxSelectMultiple,
            required=False,
        )
        status = forms.MultipleChoiceField(
            choices=[("active", _("Active")), ("inactive", _("Inactive"))],
            widget=forms.CheckboxSelectMultiple,
            required=False,
        )
        trash = forms.BooleanField(required=False, label=_("Trash"))

    def get_layout(self):
        self.checkboxcounterid = hg.html_id(self, "checkbox-counter")
        ret = super().get_layout()
        toolbar = list(
            ret.filter(
                lambda e, a: getattr(e, "attributes", {}).get("_class", "")
                == "bx--toolbar-content"
            )
        )[0]
        nfilters = self._checkbox_count()
        toolbar.insert(
            -2,
            hg.DIV(
                hg.SPAN(nfilters, id=self.checkboxcounterid),
                layout.icon.Icon(
                    "close",
                    focusable="false",
                    size=15,
                    role="img",
                    onclick=f"document.location = '{self.request.path}?reset=1'",
                ),
                role="button",
                _class="bx--list-box__selection bx--list-box__selection--multi bx--tag--filter",
                style="margin: auto 0.5rem;"
                + (" display: none;" if nfilters == 0 else ""),
                tabindex="0",
                title=("Reset"),
            ),
        )
        return ret

    def _filterform(self):
        return self.FilterForm({"status": ["active"], **self.request.GET})

    def _checkbox_count(self):
        counter = 0
        form = self._filterform()
        if form.is_valid():
            counter += 1 if form.cleaned_data["naturalperson"] else 0
            counter += 1 if form.cleaned_data["legalperson"] else 0
            counter += 1 if form.cleaned_data["personassociation"] else 0
            counter += form.cleaned_data["naturalperson_subtypes"].count()
            counter += form.cleaned_data["legalperson_subtypes"].count()
            counter += form.cleaned_data["personassociation_subtypes"].count()
            counter += form.cleaned_data["tags"].count()
            counter += len(form.cleaned_data["preferred_language"])
            counter += (
                1 if "inactive" in form.cleaned_data["status"] else 0
            )  # don't count the "active" checkbox, it is a permanent default
            counter += 1 if form.cleaned_data["trash"] else 0
        return counter

    def get_queryset(self):
        form = self._filterform()
        qs = super().get_queryset()
        if form.is_valid():
            ret = (
                (qs.filter(deleted=form.cleaned_data.get("trash", False)))
                .select_related(
                    "primary_email_address", "primary_postal_address", "_type"
                )
                .prefetch_related("tags")
            )
            if any(
                [
                    form.cleaned_data[i]
                    for i in (
                        "naturalperson",
                        "legalperson",
                        "personassociation",
                        "naturalperson_subtypes",
                        "legalperson_subtypes",
                        "personassociation_subtypes",
                    )
                ]
            ):
                q = Q()
                for i in ("naturalperson", "legalperson", "personassociation"):
                    # setup some logic descriptors
                    maintype_selected = bool(form.cleaned_data[i])
                    subtype_selected = bool(form.cleaned_data[f"{i}_subtypes"])
                    all_subtypes_selected = bool(
                        form.cleaned_data[f"{i}_subtypes"].count()
                        == form.fields[f"{i}_subtypes"].queryset.count()
                    )

                    # the semantics for this filter are not 100% clear
                    # there are also cases where a subtype has the wrong maintype
                    # This code tries to make the selection consistent to what a user
                    # would expect, but these expectations can still vary...
                    if maintype_selected:
                        typeq = Q(_maintype=i)
                        if subtype_selected:
                            if not all_subtypes_selected:
                                typeq &= Q(_type__in=form.cleaned_data[f"{i}_subtypes"])
                        else:
                            typeq &= ~Q(_type__in=form.fields[f"{i}_subtypes"].queryset)
                        q |= typeq
                    else:
                        q |= Q(_type__in=form.cleaned_data[f"{i}_subtypes"])
                ret = ret.filter(q)
            if form.cleaned_data.get("tags"):
                ret = ret.filter(tags__in=form.cleaned_data["tags"])
            if form.cleaned_data.get("preferred_language"):
                ret = ret.filter(
                    preferred_language__in=form.cleaned_data["preferred_language"]
                )
            if len(form.cleaned_data.get("status")) == 1 and not form.cleaned_data.get(
                "trash", False
            ):
                ret = ret.filter(active=form.cleaned_data.get("status")[0] == "active")

            return ret
        return qs

    def get_settingspanel(self):
        return hg.DIV(
            layout.forms.Form(
                self._filterform(),
                hg.DIV(
                    hg.DIV(
                        hg.DIV(layout.helpers.Label(_("Person Type"))),
                        hg.DIV(
                            hg.DIV(
                                hg.DIV(
                                    layout.forms.FormField(
                                        "naturalperson",
                                        onclick="updateCheckboxGroupItems(this.parentElement.parentElement)",
                                    ),
                                    hg.DIV(
                                        layout.forms.FormField(
                                            "naturalperson_subtypes",
                                            style="padding-left: 1rem",
                                        ),
                                        style="margin-top: -2rem",
                                    ),
                                ),
                                layout.forms.FormField(
                                    "personassociation",
                                    onclick="updateCheckboxGroupItems(this.parentElement.parentElement)",
                                ),
                                hg.DIV(
                                    layout.forms.FormField(
                                        "personassociation_subtypes",
                                        style="padding-left: 1rem",
                                    ),
                                    style="margin-top: -2rem",
                                ),
                                style="margin-right: 16px",
                            ),
                            hg.DIV(
                                layout.forms.FormField(
                                    "legalperson",
                                    onclick="updateCheckboxGroupItems(this.parentElement.parentElement)",
                                ),
                                hg.DIV(
                                    layout.forms.FormField(
                                        "legalperson_subtypes",
                                        style="padding-left: 1rem",
                                    ),
                                    style="margin-top: -2rem",
                                ),
                                style="margin-right: 16px",
                            ),
                            style="display: flex",
                        ),
                        style="border-right: #ccc solid 1px; margin: 0 16px 0 0",
                    ),
                    hg.DIV(
                        hg.DIV(
                            layout.forms.FormField("tags"),
                            style="margin-right: 16px",
                        ),
                        style="border-right: #ccc solid 1px; margin: 0 16px 0 0; overflow-y: scroll",
                    ),
                    hg.DIV(
                        hg.DIV(
                            layout.forms.FormField("preferred_language"),
                            style="margin-right: 16px",
                        ),
                        style="border-right: #ccc solid 1px; margin: 0 16px 0 0",
                    ),
                    hg.DIV(
                        hg.DIV(layout.forms.FormField("status"), style="flex-grow: 0"),
                        hg.DIV(style="flex-grow: 1"),
                        hg.DIV(
                            layout.forms.FormField("trash"),
                            style="max-height: 2rem",
                        ),
                        style="display: flex; flex-direction: column",
                    ),
                    style="display: flex; max-height: 50vh; padding: 24px 32px 0 32px",
                ),
                hg.DIV(
                    layout.button.Button(
                        _("Cancel"),
                        buttontype="ghost",
                        onclick="this.parentElement.parentElement.parentElement.parentElement.parentElement.style.display = 'none'",
                    ),
                    layout.button.Button.from_link(
                        Link(
                            label=_("Reset"),
                            href=self.request.path + "?reset=1",
                            iconname=None,
                        ),
                        buttontype="secondary",
                    ),
                    layout.button.Button(
                        pgettext_lazy("apply filter", "Filter"),
                        type="submit",
                    ),
                    style="display: flex; justify-content: flex-end; margin-top: 24px",
                    _class="bx--modal-footer",
                ),
                method="GET",
            ),
            hg.SCRIPT(
                mark_safe(
                    """
                    function updateCheckboxGroupItems(group) {
                        var items = $$('input[type=checkbox]', group);
                        var value = items[0].getAttribute('aria-checked');
                        value = value == 'true' ? 'true' : 'false';
                        for(var i = 1; i < items.length; ++i) {
                            new CarbonComponents.Checkbox(items[i]).setState(value);
                        }
                    }
                    function updateCheckboxCounter(group) {
                        var items = $$('input[type=checkbox]', group);
                        var count = 0;
                        for(item of items) {
                            if(!(item.name === 'status' && item.value === 'active'))
                                count += item.getAttribute('aria-checked') == 'true' ? 1 : 0
                        }
                        $('#%s').innerHTML = count;
                        $('#%s').closest('div[role=button]').style.display = count === 0 ? "none" : "flex";
                    }
                    """
                    % (self.checkboxcounterid, self.checkboxcounterid)
                )
            ),
            style="background-color: #fff",
            onclick="updateCheckboxCounter(this)",
        )



import htmlgenerator as hg
from bread.utils import Link, ModelHref
from bread.views import BrowseView
from django.utils.translation import gettext_lazy as _

from basxconnect.core.models import Term


class VocabularyBrowseView(BrowseView):
    rowactions = [
        BrowseView.editlink(),
        Link(
            href=ModelHref(Term, "browse", query={"vocabulary_slug": hg.C("row").slug}),
            label=_("Terms of vocabulary"),
            iconname="tree-view--alt",
        ),
    ]



from django.utils.html import mark_safe
from dynamic_preferences.registries import global_preferences_registry

from basxconnect.core.views.person.search_person_view import searchbar


def basxconnect_core(request):
    return {
        "PLATFORMNAME": mark_safe('basx <span style="font-weight: 600">Connect</span>'),
        "COMPANYNAME": global_preferences_registry.manager()[
            "general__organizationname"
        ],
        "SEARCHBAR": searchbar,
    }



from typing import Callable

import htmlgenerator as hg
from bread import layout
from bread.layout.components.datatable import DataTableColumn
from bread.utils import Link, ModelHref, reverse_model
from django.shortcuts import get_object_or_404
from django.utils.translation import gettext_lazy as _

from basxconnect.core.layouts.editperson.common import utils
from basxconnect.core.models import Person, Relationship

R = layout.grid.Row
C = layout.grid.Col


def relationshipstab(request):
    person = get_object_or_404(Person, pk=request.resolver_match.kwargs["pk"])
    modal_from = modal_add_relationship_from(person)
    modal_to = modal_add_relationship_to(person)
    return layout.tabs.Tab(
        _("Relationships"),
        utils.grid_inside_tab(
            R(
                utils.tiling_col(
                    relationships_datatable(
                        request,
                        title=_("Relationships to person"),
                        queryset=hg.F(lambda c: c["object"].relationships_from.all()),
                        primary_button=button_add_relationship_to(modal_to),
                    ),
                    modal_to,
                    hg.DIV(style="margin-top: 4rem;"),
                    relationships_datatable(
                        request,
                        title=_("Relationships from person"),
                        queryset=hg.F(lambda c: c["object"].relationships_to.all()),
                        primary_button=button_add_relationship_from(modal_from),
                    ),
                    modal_from,
                )
            ),
        ),
    )


def relationships_datatable(request, queryset, primary_button, title):
    return layout.datatable.DataTable.from_queryset(
        queryset,
        model=Relationship,
        title=title,
        prevent_automatic_sortingnames=True,
        columns=[
            person_in_relationship(
                "Person A",
                "person_a",
                lambda relationship: relationship.person_a,
            ),
            "type",
            person_in_relationship(
                "Person B",
                "person_b",
                lambda relationship: relationship.person_b,
            ),
            "start_date",
            "end_date",
        ],
        rowactions=[
            Link(
                href=ModelHref(
                    Relationship,
                    "edit",
                    kwargs={"pk": hg.C("row.pk")},
                    query={"next": request.get_full_path()},
                ),
                iconname="edit",
                label=_("Edit"),
            ),
            Link(
                href=ModelHref(
                    Relationship,
                    "delete",
                    kwargs={"pk": hg.C("row.pk")},
                    query={"next": request.get_full_path()},
                ),
                iconname="trash-can",
                label=_("Delete"),
            ),
        ],
        primary_button=primary_button,
    )


def button_add_relationship_from(modal):
    return layout.button.Button(
        _("Add relationship from person"),
        **modal.openerattributes,
    )


def button_add_relationship_to(modal):
    return layout.button.Button(
        _("Add relationship to person"),
        **modal.openerattributes,
    )


def modal_add_relationship_from(person):
    ret = layout.modal.Modal.with_ajax_content(
        heading=_("Add relationship from person"),
        url=ModelHref(
            Relationship, "add", query={"asajax": True, "person_a": person.pk}
        ),
        submitlabel=_("Save"),
    )
    return ret


def modal_add_relationship_to(person):
    ret = layout.modal.Modal.with_ajax_content(
        heading=_("Add relationship to person"),
        url=ModelHref(
            Relationship, "add", query={"asajax": True, "person_b": person.pk}
        ),
        submitlabel=_("Save"),
    )
    return ret


def person_in_relationship(
    header: str,
    field_name: str,
    get_person: Callable[[Relationship], Person],
) -> DataTableColumn:
    return DataTableColumn(
        header,
        hg.SPAN(
            person_name(field_name),
            person_number_in_brackets(field_name),
            **attributes_for_link_to_person(get_person),
        ),
    )


def attributes_for_link_to_person(get_person: Callable[[Relationship], Person]):
    return layout.aslink_attributes(
        hg.F(
            lambda c: reverse_model(
                get_person(c["row"]),
                "read",
                kwargs={"pk": get_person(c["row"]).pk},
            )
        )
    )


def person_name(field):
    return hg.C(f"row.{field}")


def person_number_in_brackets(field):
    return hg.SPAN(" [", hg.C(f"row.{field}.personnumber"), "]")



import htmlgenerator as hg
from bread import layout
from bread.layout.components.icon import Icon
from django.utils.translation import gettext_lazy as _

from basxconnect.core import models
from basxconnect.core.layouts.editperson.common.base_data import common_tiles
from basxconnect.core.layouts.editperson.common.utils import (
    grid_inside_tab,
    person_metadata,
    tile_col_edit_modal,
)

R = layout.grid.Row
C = layout.grid.Col


def base_data_tab(request):
    return layout.tabs.Tab(
        _("Base data"),
        hg.BaseElement(
            grid_inside_tab(
                R(
                    tile_col_edit_modal(
                        _("Base Data"),
                        models.LegalPerson,
                        "ajax_edit_personal_data",
                        Icon("building"),
                        [
                            "name",
                            "name_addition",
                        ],
                    ),
                    person_metadata(models.LegalPerson),
                ),
                common_tiles(request),
            ),
        ),
    )



import htmlgenerator as hg
from bread import layout as layout
from bread.utils import reverse_model
from django.http import HttpResponse
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from haystack.query import SearchQuerySet
from haystack.utils import Highlighter
from htmlgenerator import mark_safe

from ... import models, settings

R = layout.grid.Row
C = layout.grid.Col

ITEM_CLASS = "search_result_item"
ITEM_LABEL_CLASS = "search_result_label"
ITEM_VALUE_CLASS = "search_result_value"

searchbar = layout.search.Search(
    placeholder=_("Search person"),
    backend=layout.search.SearchBackendConfig(
        reverse_lazy("basxconnect.core.views.person.search_person_view.searchperson"),
    ),
    width="140%",
)


def searchperson(request):
    query = request.GET.get("q")

    highlight = CustomHighlighter(query)
    if not query or len(query) < settings.MIN_CHARACTERS_DYNAMIC_SEARCH:
        return HttpResponse("")

    query_set = (
        SearchQuerySet()
        .models(models.Person)
        .autocomplete(name_auto=query)
        .filter_or(personnumber=query)
    )

    def onclick(person):
        link = reverse_model(
            person,
            "edit",
            kwargs={"pk": person.pk},
        )
        return f"document.location = '{link}'"

    ret = _display_results(query_set, highlight, onclick)
    return HttpResponse(
        hg.DIV(
            ret,
            _class="raised",
            style="margin-bottom: 1rem; padding: 16px 0 48px 48px; background-color: #fff",
        ).render({})
    )


def _display_results(query_set, highlight, onclick):
    if query_set.count() == 0:
        return _("No results")

    def _display_as_list_item(person):
        if person is None:
            # this happens if we have entries in the search-backend which have been deleted
            return hg.BaseElement()
        return hg.LI(
            hg.SPAN(
                mark_safe(highlight.highlight(person.personnumber)),
                style="width: 48px; display: inline-block",
                _class=ITEM_VALUE_CLASS,
            ),
            hg.SPAN(
                person.name,
                _class=ITEM_LABEL_CLASS,
                style="dispay:none;",
            ),
            " ",
            mark_safe(highlight.highlight(person.search_index_snippet())),
            style="cursor: pointer; padding: 8px 0;",
            onclick=onclick(person),
            onmouseenter="this.style.backgroundColor = 'lightgray'",
            onmouseleave="this.style.backgroundColor = 'initial'",
            _class=ITEM_CLASS,
        )

    result_list = [
        _display_as_list_item(search_result.object)
        for search_result in query_set[:25]
        if search_result and search_result.object
    ]

    return hg.UL(
        hg.LI(_("%s items found") % len(query_set), style="margin-bottom: 20px"),
        *result_list,
    )


class CustomHighlighter(Highlighter):
    def find_window(self, highlight_locations):
        return (0, self.max_length)



# Generated by Django 3.1.5 on 2021-03-11 17:16

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0005_auto_20210311_1851"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalnaturalperson",
            name="type",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                limit_choices_to={"category__slug": "naturaltype"},
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="core.term",
            ),
        ),
        migrations.AddField(
            model_name="naturalperson",
            name="type",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"category__slug": "naturaltype"},
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="core.term",
            ),
        ),
    ]



import django.forms
import htmlgenerator as hg
from bread import layout
from bread.layout.components.icon import Icon
from bread.views import AddView, EditView
from django.apps import apps
from django.utils.translation import gettext_lazy as _

from basxconnect.core import models


class NaturalPersonEditMailingsView(EditView):
    model = models.NaturalPerson
    fields = [
        "preferred_language",
        "type",
        "salutation_letter",
        "gender",
        "form_of_address",
    ]

    @staticmethod
    def path():
        return "person_modals_views.naturalpersoneditmailingsview"

    @staticmethod
    def read_heading():
        return _("Settings")

    @staticmethod
    def edit_heading():
        return _("Edit mailing settings")

    @staticmethod
    def icon():
        return Icon("settings--adjust")


class NaturalPersonEditPersonalDataView(EditView):
    model = models.NaturalPerson
    fields = [
        "salutation",
        "title",
        "name",
        "autogenerate_displayname",
        "first_name",
        "last_name",
        "gender",
        "date_of_birth",
        "profession",
        "deceased",
        "decease_date",
    ]

    def get_form(self, *args, **kwargs):
        form = super().get_form(*args, **kwargs)
        # only set the disabled attribute on the widget, otherwise we cannot change it on the client side in the future
        if self.object.autogenerate_displayname:
            form.fields["name"].widget.attrs["disabled"] = True
        form.fields["autogenerate_displayname"].widget.attrs[
            "onchange"
        ] = "if(this.checked) { $('input[name=name]').setAttribute('disabled', '') } else { $('input[name=name]').removeAttribute('disabled') }"
        return form

    def get_layout(self):
        R = layout.grid.Row
        C = layout.grid.Col
        F = layout.forms.FormField
        return layout.grid.Grid(
            layout.components.forms.Form(
                hg.C("form"),
                R(C(F("salutation"))),
                R(C(F("title"))),
                R(
                    C(
                        F("name"),
                        width=10,
                        breakpoint="lg",
                    ),
                    C(
                        F("autogenerate_displayname"),
                        width=6,
                        breakpoint="lg",
                        style="align-self: flex-end;",
                    ),
                ),
                R(C(F("first_name"))),
                R(C(F("last_name"))),
                R(C(F("gender"))),
                R(C(F("date_of_birth"))),
                R(C(F("profession"))),
                R(C(F("deceased"))),
                R(C(F("decease_date"))),
            )
        )


class LegalPersonEditPersonalDataView(EditView):
    model = models.LegalPerson
    fields = [
        "name",
        "name_addition",
    ]


class PersonAssociationEditPersonalDataView(EditView):
    model = models.PersonAssociation
    fields = [
        "name",
        "preferred_language",
        "salutation_letter",
    ]


class LegalPersonEditMailingsView(EditView):
    model = models.LegalPerson
    fields = [
        "preferred_language",
        "type",
        "salutation_letter",
    ]


class EditPostalAddressView(EditView):
    model = models.Postal
    fields = [
        "type",
        "address",
        "postcode",
        "city",
        "country",
        "valid_from",
        "valid_until",
    ]

    def form_valid(self, form, *args, **kwargs):
        ret = super().form_valid(form, *args, **kwargs)
        is_primary = form.cleaned_data["is_primary"]
        if is_primary:
            self.object.person.primary_postal_address = self.object
        self.object.person.save()
        return ret

    def get_form_class(self, *args, **kwargs):
        class EditPostalForm(super().get_form_class(*args, **kwargs)):
            is_primary = django.forms.BooleanField(
                label=_("Use as primary postal address"), required=False
            )

        return EditPostalForm

    def get_layout(self):
        form_fields = [layout.forms.FormField(field) for field in [*self.fields]] + [
            hg.If(
                hg.F(
                    lambda c: c["object"].person.primary_postal_address
                    and c["object"].person.primary_postal_address.pk != c["object"].pk
                ),
                layout.forms.FormField("is_primary"),
                "",
            )
        ]
        return hg.DIV(layout.components.forms.Form(hg.C("form"), *form_fields))


class AddPostalAddressView(AddView):
    model = models.Postal

    def form_valid(self, request, *args, **kwargs):
        ret = super().form_valid(request, *args, **kwargs)
        self.object.person.save()
        return ret


class EditEmailAddressView(EditView):
    model = models.Email
    fields = ["type", "email"]

    def form_valid(self, form, *args, **kwargs):

        ret = super().form_valid(form, *args, **kwargs)
        is_primary = form.cleaned_data["is_primary"]
        if is_primary:
            self.object.person.primary_email_address = self.object
        self.object.person.save()

        if apps.is_installed("basxconnect.mailer_integration"):
            from basxconnect.mailer_integration import settings

            new_email = form.cleaned_data["email"]
            propagate_change_to_mailer = (
                form.cleaned_data["propagate_change_to_mailer"]
                and new_email != self.old_email
                and settings.MAILER.email_exists(self.old_email)
                and not settings.MAILER.email_exists(new_email)
            )
            if propagate_change_to_mailer:
                settings.MAILER.change_email_address(self.old_email, new_email)
            elif new_email != self.old_email and hasattr(self.object, "subscription"):
                # if an email-change is not being pushed to the mailer, the subscription is likely to be outdated
                self.object.subscription.delete()
        return ret

    def post(self, request, *args, **kwargs):
        self.old_email = self.get_object().email
        return super().post(request, *args, **kwargs)

    def get_form_class(self, *args, **kwargs):
        class EditEmailForm(super().get_form_class(*args, **kwargs)):
            is_primary = django.forms.BooleanField(
                label=_("Use as primary email address"), required=False
            )
            propagate_change_to_mailer = django.forms.BooleanField(required=False)

            if apps.is_installed("basxconnect.mailer_integration"):
                import basxconnect.mailer_integration.settings as mailer_integration_settings

                propagate_change_to_mailer.label = (
                    _("Propagate change of email address to %s")
                    % mailer_integration_settings.MAILER.name()
                )

        return EditEmailForm

    def get_layout(self):
        form_fields = [layout.forms.FormField(field) for field in [*self.fields]] + [
            hg.If(
                hg.F(
                    lambda c: c["object"].person.primary_email_address
                    and c["object"].person.primary_email_address.pk != c["object"].pk
                ),
                layout.forms.FormField("is_primary"),
                "",
            ),
            hg.If(
                hg.F(
                    lambda c: apps.is_installed("basxconnect.mailer_integration")
                    and hasattr(c["object"], "subscription")
                ),
                layout.forms.FormField("propagate_change_to_mailer"),
                "",
            ),
        ]
        if self.ajax_urlparameter in self.request.GET:
            return layout.forms.Form(hg.C("form"), *form_fields)

        return layout.grid.Grid(
            hg.H3(_("Edit Email")),
            layout.grid.Row(
                layout.grid.Col(
                    layout.forms.Form(
                        hg.C("form"),
                        hg.DIV(*form_fields),
                        layout.forms.helpers.Submit(),
                    ),
                    width=4,
                )
            ),
            gutter=False,
        )


class AddEmailAddressView(AddView):
    model = models.Email

    def post(self, request, *args, **kwargs):
        ret = super().post(request, *args, **kwargs)
        self.object.person.save()
        return ret



# Generated by Django 3.2.4 on 2021-10-05 14:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core", "0019_alter_term_slug"),
    ]

    operations = [
        migrations.AddField(
            model_name="postal",
            name="valid_from",
            field=models.DateField(blank=True, null=True, verbose_name="Valid from"),
        ),
        migrations.AddField(
            model_name="postal",
            name="valid_to",
            field=models.DateField(blank=True, null=True, verbose_name="Valid to"),
        ),
    ]



from bread.tests.helper import generic_bread_testcase
from hypothesis.extra.django import from_model

from basxconnect.core import models


class VocabularyTest(generic_bread_testcase(models.Vocabulary)):
    pass


class TermTest(
    generic_bread_testcase(models.Term, vocabulary=from_model(models.Vocabulary))
):
    pass


class PersonTest(generic_bread_testcase(models.Person)):
    pass


class NaturalPersonTest(generic_bread_testcase(models.NaturalPerson)):
    pass


class LegalPersonTest(generic_bread_testcase(models.LegalPerson)):
    pass


class PersonAssociationTest(generic_bread_testcase(models.PersonAssociation)):
    pass



from bread import layout
from django.utils.translation import gettext_lazy as _

from basxconnect.core.layouts.editperson.common.utils import grid_inside_tab, tiling_col

R = layout.grid.Row


def mailings_tab(request):

    from django.apps import apps

    if apps.is_installed("basxconnect.mailer_integration"):
        from basxconnect.mailer_integration.layouts import mailer_integration_tile

        mailer_tile = mailer_integration_tile(request)
    else:
        mailer_tile = tiling_col()

    return layout.tabs.Tab(
        _("Mailings"),
        grid_inside_tab(
            R(
                mailer_tile,
                tiling_col(),
            ),
        ),
    )
